#ifndef BLOCKIO5068_APIS_H
#define BLOCKIO5068_APIS_H

#include "CommDef.h"
#include "HandleDef.h"
#include "ErrorCodeDef.h"

#if !defined(_BIO_APIS_H_)
    #define FUNCEXPORTS  __declspec (dllimport)
#else
    #define FUNCEXPORTS  __declspec (dllexport)
#endif

#ifdef __cplusplus
extern "C"
{
#endif

FUNCEXPORTS DWORD WINAPI DIO_SetDefault(HANDLE Handle);

#define I5068_DI_NUM	14
#define I5068_DO_NUM	8

FUNCEXPORTS DWORD WINAPI DIO_GetDIO(HANDLE Handle, BYTE *pbDIs, BYTE *pbDOs);
FUNCEXPORTS DWORD WINAPI DIO_SetDOs(HANDLE Handle, BYTE bDOs);
FUNCEXPORTS DWORD WINAPI DIO_SetDO(HANDLE Handle, BYTE byBitPos, BOOL bDO);

#define DI_MODE_DI			0
#define DI_MODE_COUNTER		1

FUNCEXPORTS DWORD WINAPI DIO_GetDIMode(HANDLE Handle, WORD *pwMode);
FUNCEXPORTS DWORD WINAPI DIO_SetDIMode(HANDLE Handle, WORD wMode);

#define DI_COUNTER_STATUS			0
#define DI_COUNTER_OVER_STATUS		1
#define DI_COUNTER_TRIGGER_MODE		2
#define DI_COUNTER_POWERON_STATUS	3
#define DI_COUNTER_SAFE_STATUS		4
#define DI_COUNTER_CLEAR_VALUE		5

#define DI_INFO_WORD_COUNT			6

typedef struct tagI5068CntInfo
{
	WORD  wValues[DI_INFO_WORD_COUNT];

	BYTE  byMask;		// for write
	BYTE  byResult;		// for write

} I5068_COUNTER_INFO;

FUNCEXPORTS DWORD WINAPI DIO_GetCounterInfo(HANDLE Handle, I5068_COUNTER_INFO* ptCntInfo);
FUNCEXPORTS DWORD WINAPI DIO_SetCounterInfo(HANDLE Handle, I5068_COUNTER_INFO* ptCntInfo);

// byChId: 0 ~ 13, 0xff - all
FUNCEXPORTS DWORD WINAPI DIO_GetCounterValue(HANDLE Handle, BYTE byChId, DWORD *pdwValue);

#define DO_MODE_DO			0
#define DO_MODE_PULSE		1

FUNCEXPORTS DWORD WINAPI DIO_GetDOMode(HANDLE Handle, WORD *pwMode);
FUNCEXPORTS DWORD WINAPI DIO_SetDOMode(HANDLE Handle, WORD wMode);


#define DO_PULSE_STATUS				0
#define DO_PULSE_POWERON_STATUS		1
#define DO_PULSE_SAFE_STATUS		2
#define DO_POWERON_STATUS			3
#define DO_SAFE_STATUS				4

#define DO_INFO_WORD_COUNT			5

typedef struct tagI5068PulseInfo
{
	WORD  wValues[DO_INFO_WORD_COUNT];

	BYTE  byMask;		// for write
	BYTE  byResult;		

} I5068_PULSE_INFO;

FUNCEXPORTS DWORD WINAPI DIO_GetPulseInfo(HANDLE Handle, I5068_PULSE_INFO* ptPulseInfo);
FUNCEXPORTS DWORD WINAPI DIO_SetPulseInfo(HANDLE Handle, I5068_PULSE_INFO* ptPulseInfo);

// byChId: 0 ~ 7
FUNCEXPORTS DWORD WINAPI DIO_GetPulseOutputCount(HANDLE Handle, BYTE byChId, DWORD* pdwCount);
FUNCEXPORTS DWORD WINAPI DIO_SetPulseOutputCount(HANDLE Handle, BYTE byChId, DWORD dwCount);

// byChId: 0 ~ 7
// pdwFreq: LoWord - low signal width
//			HiWord - high signal width
FUNCEXPORTS DWORD WINAPI DIO_GetPulseFreq(HANDLE Handle, BYTE byChId, DWORD* pdwFreq);

// nType : 0 - low  1 - high  2 - all
// dwFreq: LoWord   LoWord    LoWord+HiWord
FUNCEXPORTS DWORD WINAPI DIO_SetPulseFreq(HANDLE Handle, BYTE byChId, int nType, DWORD dwFreq);

FUNCEXPORTS DWORD WINAPI DIO_SetSnmpTrapCount(HANDLE Handle, BYTE byCount);
FUNCEXPORTS DWORD WINAPI DIO_GetSnmpTrapCount(HANDLE Handle, BYTE *pbyCount);
FUNCEXPORTS DWORD WINAPI DIO_SetSnmpTrapIP(HANDLE Handle, BYTE byChId, LPCTSTR lpszIP);
FUNCEXPORTS DWORD WINAPI DIO_GetSnmpTrapIP(HANDLE Handle, BYTE byChId, LPTSTR lpszIP);
FUNCEXPORTS DWORD WINAPI DIO_ResetSnmpTrap(HANDLE Handle);
FUNCEXPORTS DWORD WINAPI DIO_SetSnmpTrapIpHelper(HANDLE hSession, BYTE byChId, LPCTSTR lpszIP, int nOffset);
FUNCEXPORTS DWORD WINAPI DIO_GetSnmpTrapIpHelper(HANDLE hSession, BYTE byChId, LPTSTR lpszIP, int nOffset);

FUNCEXPORTS DWORD WINAPI DIO_SetSafetyDO(HANDLE hSession);
FUNCEXPORTS DWORD WINAPI DIO_SetPowerOnDO(HANDLE hSession);

#define I5068_RULE_COUNT		8
#define I5068_RULE_CONDI_COUNT	4

typedef struct tagI5068LogicRuleIF
{
	// if (DI op action) then DO action
	DWORD dwSetCntValue;  // Set Counter Value

	BYTE byDICh;    // 0 ~ (I5068_DI_NUM-1)

	BYTE byOp;	    // 0:On 1:Off 2:Off-On 3:On-Off 4:= 5:> 6:< 7:>= 8:<=

	BYTE byFlag;	// bit 0: disable/enable [bit 1: AI:1 DI:0 (5815)]
	//BYTE dummy[3];
	WORD wAIValue;	// (5815)
	BYTE dummy;

} I5068_LOGIC_RULE_IF;

#define LOGIC_ACT_ON			0
#define LOGIC_ACT_OFF			1
#define LOGIC_ACT_CNT_RESET		2
#define LOGIC_ACT_PLS_START		3
#define LOGIC_ACT_PLS_STOP		4
#define LOGIC_ACT_SNMP_TRAP		5
#define LOGIC_ACT_AO_OUT		6

#define LOGIC_RULE_REL_OR		0
#define LOGIC_RULE_REL_AND		1

typedef struct tagI5068LogicRuleTHEN
{
	BYTE byDIOCh;     // DI or DO Channel
	BYTE byAct;	      // 0 ~ 5
					  // 0:On 1:Off 2:Counter reset 3:Pulse out start 4:Pulse out stop 
					  // 5:Snmp trap [6:AO Output (5815)]
	BYTE byFlag;	  // bit 0: disable/enable
	WORD wSetAOValue; // AO Output Value (5815)  
	BYTE dummy[3];    // 쥻uBYTE dummy{bsWWORD wSetAOValueMBYTE dummy[3]

} I5068_LOGIC_RULE_THEN;

typedef struct tagI5068LogicRule
{
	I5068_LOGIC_RULE_IF tIf[I5068_RULE_CONDI_COUNT];
	I5068_LOGIC_RULE_THEN tThen[I5068_RULE_CONDI_COUNT];

	WORD wChkTime;    // 0 ~ 65535
	WORD wRelIf;	  // 0:OR 1:AND
	BYTE byFlag;	  // bit 0: disable/enable [bit 1: 0:Latch 1:Monentary (5815)]
	BYTE dummy[3];

} I5068_LOGIC_RULE;

// \/\/: 
// byRuleId : 0 ~ I5068_RULE_COUNT-1 or 0xff
FUNCEXPORTS DWORD WINAPI DIO_GetLogicRules(HANDLE Handle, BYTE byRuleId, I5068_LOGIC_RULE *pRules);

// byRuleId : 0 ~ I5068_RULE_COUNT-1 or 0xff
FUNCEXPORTS DWORD WINAPI DIO_SetLogicRules(HANDLE Handle, BYTE byRuleId, I5068_LOGIC_RULE *pRules);
// /\/\:

// /\/\:

// \/\/:  PPIO Service
#define I5068_PPIO_FIRMVER	0xF205

#define I5068_PPIO_RULE_COUNT	7

#define I5068_PPIO_ENABLED	0x000001	// 0: disabled 1: enabled
#define I5068_PPIO_ROLE		0x000002	// 0: client 1: server

typedef struct tagI5068PPIOConnectOpt
{
	DWORD   dwSize;			// sizeof(I5068_PPIO_CONNECT_OPT)
	DWORD   dwFlags;		// See above I5068_PPIO_XXX
	LPTSTR  lpszAddr;		// Mirror to / Mirror from
	
} I5068_PPIO_CONNECT_OPT;

typedef struct tagI5068PPIOValue
{
	DWORD dwSize;			// sizeof(I5068_PPIO_VALUE)
	WORD  wDInp;
	WORD  wDOut;

} I5068_PPIO_VALUE;

typedef struct tagI5068PPIOStatus
{
	DWORD dwSize;			// sizeof(I5068_PPIO_STATUS)
	WORD  wDeviceId;		// 0x5068 OR 0x5068
	WORD  wActive;			// 1: connect 0: not connect

} I5068_PPIO_STATUS;

#define I5068_PPIO_TYPE_CONNECT_OPT		1
#define I5068_PPIO_TYPE_VAULE			2
#define I5068_PPIO_TYPE_STATUS			3

// nType : See above I5068_PPIO_TYPE_XXX
FUNCEXPORTS DWORD WINAPI DIO_SetPPIO(HANDLE Handle, int nType, void *pOpts);
FUNCEXPORTS DWORD WINAPI DIO_GetPPIO(HANDLE Handle, int nType, void *pOpts);

// /\/\:

#ifdef __cplusplus
}
#endif

#endif