#ifndef ERROR_CODE_DEF
#define ERROR_CODE_DEF

// An Error Code ID is a 32 bit value which is the combination of 
// a System ID, an AUX Code, a Target ID and Error Code. The bits 
// are allocates as follows:
//	
//	+---------+------------------+-----------+------------+
//	|System ID|     AUX Code     | Target ID | Error Code |
// 	+---------+------------------+-----------+------------+
// 	31       28 27             16 15        8 7           0 
//	
//	System ID: System Identification (0x0-0xF)
//		0: Reversed	1:				2: Remote Module
//		3:			4:				5: 
//		6:			7:				8: 
//		9:			A: Operation	

//	AUX Code: I/O, Memory Map Port or Auxillary Code (0x000-0xFFF)
//	Target ID: I/O device driver or modules (0x00-0xFF)
//
//		00: Reversed	01: 6000		02: 601x
//		03: 602x		04: 604x		
//		
//		10: Reversed    12: 6012		13: 6013
//		14: 6014		17: 6017		18: 6018
//
//		20: Reversed	21: 6021		22: 6022		
//
//		40: Reversed	42: 6042		44: 6044
//      46: 6046	    
//      
//		A0: Reversed    A1: ComPort
//
//		Error Code: Error Status (0x00-0xFF)
//	

// Target ID
#define TARGET_ID_6000		0x01
#define TARGET_ID_601X		0x02
#define TARGET_ID_602X		0x03
#define TARGET_ID_604X		0x04
#define TARGET_ID_5000		0x05
#define TARGET_ID_501X		0x06

#define TARGET_ID_6012		0x12
#define TARGET_ID_6013		0x13
#define TARGET_ID_6014		0x14
#define TARGET_ID_6017		0x17
#define TARGET_ID_6018		0x18

#define TARGET_ID_6021		0x21
#define TARGET_ID_6022		0x22
#define TARGET_ID_6024		0x24

#define TARGET_ID_6042		0x42
#define TARGET_ID_6044		0x44
#define TARGET_ID_6046		0x46

#define TARGET_ID_6811		0x81

#define TARGET_ID_6815		0x85
#define TARGET_ID_5815		0x55

#define TARGET_ID_5024		0x24
#define TARGET_ID_6520		TARGET_ID_5024
#define TARGET_ID_5068		0x68
#define TARGET_ID_6550		TARGET_ID_5068
#define TARGET_ID_5017		0x57
#define TARGET_ID_6510		TARGET_ID_5017
#define TARGET_ID_5014		0x54
#define TARGET_ID_6512		TARGET_ID_5014		
#define TARGET_ID_5018		0x58
#define TARGET_ID_6511		TARGET_ID_5018		

#define TARGET_ID_COMPORT	0xA1

// Common Error Code
#define ERR_SUCCESS			  0
#define ERR_NOT_INSTALLED	  1
#define ERR_CREATE_HANDLE	  2
#define ERR_INVALID_HANDLE	  3
#define ERR_OUT_OF_MEMORY	  4
#define ERR_INVALID_COMMAND	  5
#define ERR_TIMEOUT			  6
#define ERR_POLL			  7
#define ERR_INVALID_PARAM	  8
#define ERR_OUT_OF_SIZE		  9	
#define ERR_NOT_IMP			  10
#define ERR_INVALID_RESPONSE  11
#define ERR_OPEN			  12
#define ERR_CLOSE			  13
#define ERR_NOT_SUPPORT		  14
#define ERR_OPEN_RES		  15
#define ERR_NOT_FOUND_RES	  16
#define ERR_NO_VALID_HANDLE	  17

// Error Code ID
#define ERR_CODE_ID(s,a,t,e)	MAKELONG(MAKEWORD((e),(t)), MAKEWORD((a),(s)))

#define ERR_CODE_CARD_MODULE(t,e)	 ERR_CODE_ID(9, 0, t, e)
#define ERR_CODE_REMOTE_MODULE(t,e)  ERR_CODE_ID(2, 0, t, e)
#define ERR_CODE_COMM(e)			 ERR_CODE_ID(2, 0, TARGET_ID_COMPORT, e)

#define ERR_CODE(e) ((BYTE)((e) & 0xff))
#define ERR_OK(e) (ERR_CODE(e) == ERR_SUCCESS)
#define ERR_FAILED(e) (!ERR_OK(e))

#endif