using System;
using System.Runtime.InteropServices;
using System.Text;

namespace BlockIOSDK
{
    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    public struct I5068_COUNTER_INFO
    {
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 6)]
	    public UInt16[]  pwValues;
	    public byte  byMask;		
	    public byte  byResult;	
    }

    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    public struct I5068_PULSE_INFO
    {
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 5)]
        public UInt16[] pwValues;
        public byte byMask;		
        public byte byResult;
    }

    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    public struct I5068_LOGIC_RULE_IF
    {
        // if (DI op action) then DO action
        public ulong  dwSetCntValue;  // Set Counter Value

        public byte byDICh;    // 0 ~ (I6510_DI_NUM-1)

        public byte byOp;	    // 0:On 1:Off 2:Off-On 3:On-Off 4:= 5:> 6:< 7:>= 8:<=

        public byte byFlag;	// bit 0: disable/enable [bit 1: AI:1 DI:0]
        //BYTE dummy[3];
        public ushort  wAIValue;
        public byte  dummy;
    }

    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    public struct I5068_LOGIC_RULE_THEN
    {
	    public byte  byDIOCh;     // DI or DO Channel
	    public byte  byAct;	      // 0 ~ 5
					      // 0:On 1:Off 2:Counter reset 3:Pulse out start 4:Pulse out stop 
					      // 5:Snmp trap [6:AO Output]
	    public byte  byFlag;	  // bit 0: disable/enable
	    public UInt16  wSetAOValue; // AO Output Value  
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 3)]
	    public byte[]  dummy;    // reserve 3-bytes
    }

    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    public struct I5068_LOGIC_RULE
    {
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 4)]
	    public I5068_LOGIC_RULE_IF[] tIf;
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 4)]
	    public I5068_LOGIC_RULE_THEN[] tThen;
	    public ushort  wChkTime;    // 0 ~ 65535
	    public ushort  wRelIf;	  // 0:OR 1:AND
	    public byte  byFlag;	  // bit 0: disable/enable [bit 1: 0:Latch 1:Monentary]
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 3)]
	    public byte[] dummy;
    }

    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    struct I5068_PPIO_CONNECT_OPT
    {
	    public uint    dwSize;			// sizeof(I6510_PPIO_CONNECT_OPT)
	    public uint    dwFlags;		// See above I6510_PPIO_XXX
	    public StringBuilder  lpszAddr;		// Mirror to / Mirror from
    }

    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    struct I5068_PPIO_VALUE
    {
	    public uint  dwSize;			// sizeof(I6510_PPIO_VALUE)
	    public Int16  wDInp;
	    public Int16  wDOut;
    }

    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    struct I5068_PPIO_STATUS
    {
	    public uint  dwSize;			// sizeof(I6510_PPIO_STATUS)
	    public Int16   wDeviceId;		// 0x6510 OR 0x6510
	    public Int16   wActive;			// 1: connect 0: not connect
    }

    public class BlockIO5068r
    {
        [DllImport("BlockIO5068r.dll", EntryPoint = "DIO_SetDefault")]
        public static extern uint DIO_SetDefault(IntPtr Handle);

        [DllImport("BlockIO5068r.dll", EntryPoint = "DIO_SetDOs")]
        public static extern uint DIO_SetDOs(IntPtr Handle, byte DOs);

        [DllImport("BlockIO5068r.dll", EntryPoint = "DIO_GetDIMode")]
        public static extern uint DIO_GetDIMode(IntPtr Handle, out ushort pwMode);

        [DllImport("BlockIO5068r.dll", EntryPoint = "DIO_SetDIMode")]
        public static extern uint DIO_SetDIMode(IntPtr Handle, ushort wMode);

        [DllImport("BlockIO5068r.dll", EntryPoint = "DIO_GetCounterInfo")]
        public static extern uint DIO_GetCounterInfo(IntPtr Handle, out I5068_COUNTER_INFO ptCntInfo);

        [DllImport("BlockIO5068r.dll", EntryPoint = "DIO_SetCounterInfo")]
        public static extern uint DIO_SetCounterInfo(IntPtr Handle, ref I5068_COUNTER_INFO ptCntInfo);

        [DllImport("BlockIO5068r.dll", EntryPoint = "DIO_GetCounterValue")]
        public static extern uint DIO_GetCounterValue(IntPtr Handle, byte byChId, out uint pdwValue);

        [DllImport("BlockIO5068r.dll", EntryPoint = "DIO_GetDOMode")]
        public static extern uint DIO_GetDOMode(IntPtr Handle, out ushort pwMode);

        [DllImport("BlockIO5068r.dll", EntryPoint = "DIO_SetDOMode")]
        public static extern uint DIO_SetDOMode(IntPtr Handle, ushort wMode);

        [DllImport("BlockIO5068r.dll", EntryPoint = "DIO_GetPulseInfo")]
        public static extern uint DIO_GetPulseInfo(IntPtr Handle, out I5068_PULSE_INFO ptPulseInfo);

        [DllImport("BlockIO5068r.dll", EntryPoint = "DIO_SetPulseInfo")]
        public static extern uint DIO_SetPulseInfo(IntPtr Handle, ref I5068_PULSE_INFO ptPulseInfo);

        [DllImport("BlockIO5068r.dll", EntryPoint = "DIO_GetPulseOutputCount")]
        public static extern uint DIO_GetPulseOutputCount(IntPtr Handle, byte byChId, out uint pdwCount);

        [DllImport("BlockIO5068r.dll", EntryPoint = "DIO_SetPulseOutputCount")]
        public static extern uint DIO_SetPulseOutputCount(IntPtr Handle, byte byChId, uint dwCount);

        [DllImport("BlockIO5068r.dll", EntryPoint = "DIO_GetPulseFreq")]
        public static extern uint DIO_GetPulseFreq(IntPtr Handle, byte byChId, out uint pdwFreq);

        [DllImport("BlockIO5068r.dll", EntryPoint = "DIO_SetPulseFreq")]
        public static extern uint DIO_SetPulseFreq(IntPtr Handle, byte byChId, int nType, uint dwFreq);

        [DllImport("BlockIO5068r.dll", EntryPoint = "DIO_SetSnmpTrapCount")]
        public static extern uint DIO_SetSnmpTrapCount(IntPtr Handle, byte byCount);

        [DllImport("BlockIO5068r.dll", EntryPoint = "DIO_GetSnmpTrapCount")]
        public static extern uint DIO_GetSnmpTrapCount(IntPtr Handle, out byte pbyCount);

        [DllImport("BlockIO5068r.dll", EntryPoint = "DIO_SetSnmpTrapIP")]
        public static extern uint DIO_SetSnmpTrapIP(IntPtr Handle, byte byChId, [MarshalAs(UnmanagedType.LPTStr)]string lpszIP);

        [DllImport("BlockIO5068r.dll", EntryPoint = "DIO_GetSnmpTrapIP")]
        public static extern uint DIO_GetSnmpTrapIP(IntPtr Handle, byte byChId, [MarshalAs(UnmanagedType.LPTStr)]StringBuilder lpszIP);

        [DllImport("BlockIO5068r.dll", EntryPoint = "DIO_ResetSnmpTrap")]
        public static extern uint DIO_ResetSnmpTrap(IntPtr Handle);

        [DllImport("BlockIO5068r.dll", EntryPoint = "DIO_SetSnmpTrapIpHelper")]
        public static extern uint DIO_SetSnmpTrapIpHelper(IntPtr hSession, byte byChId, [MarshalAs(UnmanagedType.LPTStr)]string lpszIP, int nOffset);

        [DllImport("BlockIO5068r.dll", EntryPoint = "DIO_GetSnmpTrapIpHelper")]
        public static extern uint DIO_GetSnmpTrapIpHelper(IntPtr hSession, byte byChId, [MarshalAs(UnmanagedType.LPTStr)]StringBuilder lpszIP, int nOffset);

        [DllImport("BlockIO5068r.dll", EntryPoint = "DIO_GetLogicRules")]
        public static extern uint DIO_GetLogicRules(IntPtr Handle, byte byRuleId, out I5068_LOGIC_RULE pRules);

        [DllImport("BlockIO5068r.dll", EntryPoint = "DIO_SetLogicRules")]
        public static extern uint DIO_SetLogicRules(IntPtr Handle, byte byRuleId, ref I5068_LOGIC_RULE pRules);

        [DllImport("BlockIO5068r.dll", EntryPoint = "DIO_SetPPIO")]
        public static extern uint DIO_SetPPIO(IntPtr Handle, int nType, [MarshalAs(UnmanagedType.LPArray)]byte[] pOpts);

        [DllImport("BlockIO5068r.dll", EntryPoint = "DIO_GetPPIO")]
        public static extern uint DIO_GetPPIO(IntPtr Handle, int nType, [MarshalAs(UnmanagedType.LPArray)]byte[] pOpts);

        [DllImport("BlockIO5068r.dll", EntryPoint = "DIO_GetDIO")]
        public static extern uint DIO_GetDIO(IntPtr Handle, [MarshalAs(UnmanagedType.LPArray)]byte[] DIs, out byte DOs);

        [DllImport("BlockIO5068r.dll", EntryPoint = "DIO_SetDO")]
        public static extern uint DIO_SetDO(IntPtr Handle, byte BitPos, bool bDO);

        [DllImport("BlockIO5068r.dll", EntryPoint = "DIO_SetSafetyDO")]
        public static extern uint DIO_SetSafetyDO(IntPtr Handle);

        [DllImport("BlockIO5068r.dll", EntryPoint = "DIO_SetPowerOnDO")]
        public static extern uint DIO_SetPowerOnDO(IntPtr Handle);
    }
}
